@@ -80,6 +80,7 @@ class Graph(BaseMainGraph):
     self.db.execute("""PRAGMA cache_size = -200000""")
     self.db.execute("""PRAGMA mmap_size = 30000000000""")
     self.db.execute("""PRAGMA page_size = 32768""")
+    #self.db.execute("""PRAGMA analysis_limit = 1000""")
     
     if sqlite_tmp_dir:
       try: self.db.execute("""PRAGMA temp_store_directory = '%s'""" % sqlite_tmp_dir)
@@ -137,7 +138,7 @@ class Graph(BaseMainGraph):
     
     self.lock              = multiprocessing.RLock()
     self.lock_level        = 0
-    
+
     if initialize_db:
       self.current_blank    = multiprocessing.Value("i", 0)
       self.current_resource = multiprocessing.Value("i", 300) # 300 first values are reserved
@@ -169,6 +170,7 @@ class Graph(BaseMainGraph):
       self.execute("""CREATE TABLE last_numbered_iri(prefix TEXT, i INTEGER)""")
       self.execute("""CREATE INDEX index_last_numbered_iri ON last_numbered_iri(prefix)""")
       
+      self.analyze()
       self.db.commit()
       
     else:
@@ -377,14 +379,31 @@ class Graph(BaseMainGraph):
         version += 1
         
       self.prop_fts = { storid for (storid,) in self.execute("""SELECT storid FROM prop_fts;""") }
+
+      self.analyze()
       
     self.current_changes = self.db.total_changes
     self.select_abbreviate_method()
-
     
+  def analyze(self):
+    if self.read_only: return
+    nb_objs  = self.execute("""SELECT COUNT(*) FROM objs""" ).fetchone()[0]
+    nb_datas = self.execute("""SELECT COUNT(*) FROM datas""").fetchone()[0]
+    try:
+      self.execute("""DELETE FROM sqlite_stat1""")
+    except:
+      self.execute("""PRAGMA analysis_limit = 20""")
+      self.execute("""ANALYZE""")
+      self.execute("""DELETE FROM sqlite_stat1""")
+    self.execute("""INSERT INTO sqlite_stat1 VALUES
+('objs', 'index_objs_op', '%s 4 3 3 1'),
+('objs', 'index_objs_sp', '%s 3 2'),
+('datas', 'index_datas_op', '%s 4 3 3 3 1'),
+('datas', 'index_datas_sp', '%s 3 2')""" % (nb_objs, nb_objs, nb_datas, nb_datas))
+    self.execute("""ANALYZE sqlite_schema""")
     
   def set_indexed(self, indexed): pass
-    
+  
   def close(self):
     self.db.close()
     
@@ -491,26 +510,31 @@ class Graph(BaseMainGraph):
   #   self.last_numbered_iri[prefix] = 1
   #   return "%s1" % prefix
   
+  def _new_numbered_iri_2(self, prefix):
+    cur = self.execute("SELECT iri FROM resources WHERE iri GLOB ? ORDER BY LENGTH(iri) DESC, iri DESC", ("%s*" % prefix,))
+    while True:
+      iri = cur.fetchone()
+      if not iri:
+        self.execute("""INSERT INTO last_numbered_iri VALUES (?,?)""", (prefix, 1))
+        return "%s1" % prefix
+      num = iri[0][len(prefix):]
+      if num.isdigit():
+        i = int(num) + 1
+        self.execute("""INSERT INTO last_numbered_iri VALUES (?,?)""", (prefix, i))
+        return "%s%s" % (prefix, i)
+      
   def _new_numbered_iri(self, prefix):
     i = self.execute("""SELECT i FROM last_numbered_iri WHERE prefix=?""", (prefix,)).fetchone()
-    if i is None:
-      cur = self.execute("SELECT iri FROM resources WHERE iri GLOB ? ORDER BY LENGTH(iri) DESC, iri DESC", ("%s*" % prefix,))
-      while True:
-        iri = cur.fetchone()
-        if not iri: break
-        num = iri[0][len(prefix):]
-        if num.isdigit():
-          i = int(num) + 1
-          self.execute("""INSERT INTO last_numbered_iri VALUES (?,?)""", (prefix, i))
-          return "%s%s" % (prefix, i)
-        
-    else:
-      i = i[0] + 1
-      self.execute("""UPDATE last_numbered_iri SET i=? WHERE prefix=?""", (i, prefix))
-      return "%s%s" % (prefix, i)
+    if i is None: return self._new_numbered_iri_2(prefix)
+
+    i = i[0] + 1
+    iri = "%s%s" % (prefix, i)
+    if self.execute("""SELECT storid FROM resources WHERE iri=?""", (iri,)).fetchone(): # Already exists, due to a name clash, e.g. "c1" + "1" vs "c" + "11"
+      return self._new_numbered_iri_2(prefix)
+    
+    self.execute("""UPDATE last_numbered_iri SET i=? WHERE prefix=?""", (i, prefix))
+    return iri
     
-    self.execute("""INSERT INTO last_numbered_iri VALUES (?,?)""", (prefix, 1))
-    return "%s1" % prefix
   
   #def _refactor_sql(self, storid, new_iri):
   def _refactor(self, storid, new_iri):
@@ -790,7 +814,8 @@ class Graph(BaseMainGraph):
   
   def __bool__(self): return True # Reimplemented to avoid calling __len__ in this case
   def __len__(self):
-    return self.execute("SELECT COUNT() FROM quads").fetchone()[0]
+    #return self.execute("SELECT COUNT() FROM quads").fetchone()[0] # slow
+    return self.execute("SELECT COUNT() FROM objs").fetchone()[0] + self.execute("SELECT COUNT() FROM datas").fetchone()[0]
 
   
 #  def get_equivs_s_o(self, s):
@@ -1157,6 +1182,7 @@ class SubGraph(BaseSubGraph):
         cur.execute("UPDATE ontologies SET last_update=? WHERE c=?", (date, self.c,))
         
       self.parent.select_abbreviate_method()
+      self.parent.analyze()
       
       return onto_base_iri
     
@@ -1415,7 +1441,8 @@ class SubGraph(BaseSubGraph):
   def search(self, prop_vals, c = None, debug = False): return self.parent.search(prop_vals, self.c, debug)
   
   def __len__(self):
-    return self.execute("SELECT COUNT() FROM quads WHERE c=?", (self.c,)).fetchone()[0]
+    #return self.execute("SELECT COUNT() FROM quads WHERE c=?", (self.c,)).fetchone()[0] # slow
+    return self.execute("SELECT COUNT() FROM objs WHERE c=?", (self.c,)).fetchone()[0] + self.execute("SELECT COUNT() FROM datas WHERE c=?", (self.c,)).fetchone()[0]
   
   
   def _iter_ontology_iri(self, c = None):
